#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/12/19
 """
from __future__ import unicode_literals, absolute_import
from django.contrib.auth.models import User
from rest_framework import serializers
from djdg_core.exceptions import PasswordError
from djdg_core.exceptions import DifferentPasswordError


class UserSerializer(serializers.ModelSerializer):
    """
    用户序列化对象
    """
    password = serializers.CharField(allow_blank=True)

    class Meta:
        model = User

    def create(self, validated_data):
        password = validated_data.pop('password', '')
        validated_data['password'] = password
        instance = super(UserSerializer, self).create(validated_data)
        instance.set_password(password)
        instance.save()
        return instance

    def update(self, instance, validated_data):
        password = validated_data.pop('password', None)
        instance = super(UserSerializer, self).update(instance, validated_data)
        if password is not None:
            instance.set_password(password)
            instance.save()
        return instance

    def to_representation(self, instance):
        d = super(UserSerializer, self).to_representation(instance)
        d.pop('password')
        return d


class ChangePasswordUserSerializer(serializers.Serializer):
    """
    修改密码序列化对象
    """
    id = serializers.IntegerField()
    old_password = serializers.CharField()
    password = serializers.CharField()
    rep_password = serializers.CharField()

    class Meta:
        hidden_model = User
        username_field = 'username'

    def validate(self, attrs):
        user = self.meta.hidden_model.objects.get(id=attrs['id'])
        if not user.check_password(attrs['old_password']):
            raise PasswordError(getattr(user, self.meta.username_field))
        if attrs['password'] != attrs['rep_password']:
            raise DifferentPasswordError()
        return attrs

    def create(self, validated_data):
        user = self.meta.hidden_model.objects.get(id=validated_data['id'])
        user.set_password(validated_data['password'])
        user.save()
        return user



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/8/26
 """
from __future__ import unicode_literals, absolute_import
from rest_framework.permissions import BasePermission


ALLOW_ANY = 1
EXTERNAL_ONLY = 2
INTERNAL_ONLY = 3


EXTERNAL_USER_TYPE = 1
INTERNAL_USER_TYPE = 2


ALLOW_AMY_METHOD = ('get', 'options')


class SimplePermission(BasePermission):
    def has_permission(self, request, view):
        """
        action = getattr(view, request.method.lower())
        PERMISSION
                    ALLOW_ANY       允许所有类型用户访问
                    EXTERNAL_ONLY  仅客户端登录用户可以访问
                    INTERNAL_ONLY  仅内部系统app用户可以访问
        1 action 定义了 PERMISSION，则按 PERMISSION 来验证

        user_type:  1 外部用户； 2 内部系统app用户
        """
        user_type = getattr(request.user, 'USER_TYPE', None)
        method = request.method.lower()
        action = getattr(view, method)
        action_permission = getattr(action, 'PERMISSION', None)
        if action_permission is None:
            if method in ALLOW_AMY_METHOD:
                action_permission = ALLOW_ANY
            else:
                action_permission = EXTERNAL_ONLY
        if EXTERNAL_ONLY == action_permission and EXTERNAL_USER_TYPE == user_type:
            return True
        elif INTERNAL_ONLY == action_permission and INTERNAL_USER_TYPE == user_type:
            return True
        elif ALLOW_ANY == action_permission:
            return True
        return False


def permit_external_only(view):
    view.PERMISSION = EXTERNAL_ONLY
    return view


def permit_internal_only(view):
    view.PERMISSION = INTERNAL_ONLY
    return view


def allow_any(view):
    view.PERMISSION = ALLOW_ANY
    return view



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/10/19
 """
from __future__ import unicode_literals, absolute_import
from rest_framework import serializers
from djdg_core.exceptions import StanderResponseError
from djdg_core.common_api import errors


class RegisterSerializer(serializers.Serializer):
    """
    注册相关参数的校验
    """
    code = serializers.CharField(required=True)
    password = serializers.CharField(required=False)
    rep_password = serializers.CharField(required=False)
    phoneNumber = serializers.CharField(required=True)
    introducerCode = serializers.CharField(required=False)
    unionid = serializers.CharField(required=False)

    def validate(self, attrs):
        password = attrs.get('password')
        rep_password = attrs.get('rep_password')
        if password and rep_password and password != rep_password:
            raise StanderResponseError(
                errors.REPEAT_PASSWORD_NOT_RIGHT,
                '两次输入的密码不一致！'
            )
        return attrs


class LoginSerializer(serializers.Serializer):
    """
    登录相关参数的校验
    """
    phoneNumber = serializers.CharField(required=True)
    password = serializers.CharField(required=True)
    userkey = serializers.CharField(required=False)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
from djdg_core.rest_client import RestClient
from djdg_core.exceptions import StanderResponseError


class TransferStation(object):
    def __init__(self, logger, req_body, token=None, oauth_app_id=None, oauth_app_secret=None):
        """
        :param logger:
        :param req_body: {
            "method":"get",
            "host":"127.0.0.1",
            "path":"/py/v1/deal/deal-info",
            #"protocol":"http",
            #"base_url":"",
            #"params":{},
            #"json":{},
            #"action":"12"
        }
        """
        self.logger = logger
        self.req_body = req_body
        self.client = None
        self.token = token
        self.oauth_app_id = oauth_app_id
        self.oauth_app_secret = oauth_app_secret
        try:
            self.method = self.req_body.pop('method')
            self.path = self.req_body.pop('path')
            self.host = self.req_body.pop('host')
            self.base_url = self.req_body.pop('base_url') if self.req_body.get('base_url', None) else '/'
            self.protocol = self.req_body.pop('protocol') if self.req_body.get('protocol', None) else 'http'

            self.action = self.req_body.pop('action') if self.req_body.get('action', None) else None
            self.json = self.req_body.pop('json') if self.req_body.get('json', None) else None
            self.params = self.req_body.pop('params') if self.req_body.get('params', None) else None

        except Exception as e:
            msg = 'parameters error ,msg is {msg}'.format(
                msg=e.message
            )
            self.logger.info(msg)
            raise StanderResponseError(1, msg=msg)

    def init_client(self, headers=None):
        client = RestClient(
            host=self.host,
            base_url=self.base_url,
            protocol=self.protocol,
            headers=headers,
            logger=self.logger,
            oauth_app_id=self.oauth_app_id,
            oauth_app_secret=self.oauth_app_secret,
        )
        return client

    def get_result(self, client=None):
        if client:
            self.client = client
        else:
            self.client = self.init_client()
        rest_resource = self.client.get_resource(name=self.path, app=None, token=self.token)
        result = rest_resource.do_request(
            self.method,
            action=self.action,
            params=self.params,
            json=self.json,
            raise_exception=True
        )
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/10/29
 """
from __future__ import unicode_literals, absolute_import
from djdg_core.cache.redis_client import cache
from djdg_core.exceptions import TimeOutError
import time


class CacheProxy(object):
    """
    缓存代理
    """
    FORMAT = None
    LOCK_PREFIX = 'lock'
    LOCK_EXPIRE = 300
    KEY_EXPIRE = None

    def __init__(self, engine=None, **kwargs):
        if not engine:
            engine = cache
        self.__engine = engine
        self.__kwargs = kwargs
        self.__key = None
        self.__lock_key = None

    def __getattr__(self, item):
        if item.startswith('hook_'):
            item = item[5:]

            def wrapper(*args, **kwargs):
                return self.hook(item, *args, **kwargs)
            return wrapper
        return getattr(self.__engine, item)

    def hook(self, func, *args, **kwargs):
        """
        统一通过管道方式执行命令
        :param func:
        :param args:
        :param kwargs:
        :return:
        """
        expire = kwargs.pop('expire', None)
        pipe = self.__engine.pipeline()
        call_func = getattr(pipe, func)
        pipe = call_func(self.key, *args, **kwargs)
        if expire and self.KEY_EXPIRE:
            pipe = pipe.expire(self.key, self.KEY_EXPIRE)
        r = pipe.execute()
        return r[0]

    @property
    def formatter(self):
        fmts = []
        for s in self.__class__.__mro__:
            fmt = getattr(s, 'FORMAT', None)
            if fmt:
                fmts.append(fmt)
        fmts.reverse()
        return ':'.join(fmts)

    @property
    def key(self):
        if not self.__key:
            self.__key = self.formatter.format(**self.__kwargs)
        return self.__key

    @property
    def lock_key(self):
        if not self.__lock_key:
            self.__lock_key = ':'.join([self.key, self.LOCK_PREFIX])
        return self.__lock_key

    def __enter__(self):
        self.acquire()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.release()

    def acquire(self):
        start = time.time()
        while True:
            nt = time.time()
            r = self.__engine.setnx(self.lock_key, nt)
            if r:
                break
            # 检查是否超时
            v = self.__engine.get(self.lock_key)
            if v:
                fv = float(v)
                if nt - fv > self.LOCK_EXPIRE:
                    v2 = self.__engine.getset(self.lock_key, nt)
                    if v == v2:
                        break
            if nt - start > self.LOCK_EXPIRE:
                raise TimeOutError('获取锁{}超时！'.format(self.lock_key))
            time.sleep(0.1)

    def release(self):
        """
        acquire保证同时只有一个客户端获得锁，因此这里直接删除
        :return:
        """
        self.__engine.delete(self.lock_key)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2017/1/7
 """
from __future__ import unicode_literals, absolute_import
from djdg_core.apis.es import settings
from elasticsearch import Elasticsearch


es = Elasticsearch(settings.HOSTS)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2017/1/4
 """
from __future__ import unicode_literals, absolute_import
from django.conf import settings
from jpush import JPush


jg_config = settings.JPUSH


client = JPush(
    jg_config['KEY'],
    jg_config['SECRET']
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/9/30
 """
from __future__ import unicode_literals, absolute_import
from djdg_core.exceptions import ParameterNotPositiveError


class PositiveValidator(object):
    """
    正数验证器
    """
    def set_context(self, serializer_field):
        self.field_name = serializer_field.source_attrs[-1]

    def __call__(self, value):
        if 0 >= value:
            raise ParameterNotPositiveError(self.field_name)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/11/10
 """
from __future__ import unicode_literals, absolute_import


def regular_user(user):
    """
    规整用户信息
    :param user:
    :return:
    """
    if not user.get('nickname', None):
        user['nickname'] = user['username']

    if 'userKey' not in user:
        user['userKey'] = None
    return user



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/9/13
 """
from __future__ import unicode_literals, absolute_import
from rest_framework.response import Response
from rest_framework.generics import get_object_or_404
from rest_framework import mixins


class ListModelMixin(object):
    """
    List a queryset.
    """
    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        page_param = request.query_params.get('page')

        if page_param is not None:
            page = self.paginate_queryset(queryset)
            if page is not None:
                serializer = self.get_serializer(page, many=True)
                return self.get_paginated_response(serializer.data)

        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)


class RetrieveWithForeignModelMixin(object):
    """
    Retrieve a model instance.
    """
    foreign_key = ''

    def retrieve(self, request, *args, **kwargs):
        obj = self.get_queryset_with_foreign_key()
        serializer = self.get_serializer(obj, many=True)
        return Response(serializer.data)

    def get_queryset_with_foreign_key(self):
        queryset = self.filter_queryset(self.get_queryset())
        # Perform the lookup filtering.
        lookup_url_kwarg = self.lookup_url_kwarg or self.lookup_field

        assert lookup_url_kwarg in self.kwargs, (
            'Expected view %s to be called with a URL keyword argument '
            'named "%s". Fix your URL conf, or set the `.lookup_field` '
            'attribute on the view correctly.' %
            (self.__class__.__name__, lookup_url_kwarg)
        )
        foreign_key_id = '{0}_id'.format(self.foreign_key)
        filter_kwargs = {foreign_key_id: self.kwargs[lookup_url_kwarg]}
        obj = queryset.filter(**filter_kwargs)
        return obj


class LockMixin(object):
    """
    锁
    """
    def get_object(self, select_for_update=False, queryset=None):
        if not queryset:
            queryset = self.filter_queryset(self.get_queryset())
        if select_for_update:
            queryset = queryset.select_for_update()

        # Perform the lookup filtering.
        lookup_url_kwarg = self.lookup_url_kwarg or self.lookup_field

        assert lookup_url_kwarg in self.kwargs, (
            'Expected view %s to be called with a URL keyword argument '
            'named "%s". Fix your URL conf, or set the `.lookup_field` '
            'attribute on the view correctly.' %
            (self.__class__.__name__, lookup_url_kwarg)
        )

        filter_kwargs = {self.lookup_field: self.kwargs[lookup_url_kwarg]}
        obj = get_object_or_404(queryset, **filter_kwargs)

        # May raise a permission denied
        self.check_object_permissions(self.request, obj)

        return obj



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/9/18
 """
from __future__ import unicode_literals, absolute_import
from rest_framework import serializers


class SimpleListSerializer(serializers.ListSerializer):
    """
    默认的ListSerializer update操作
    """
    def update(self, instance, validated_data):
        ist_map = {ist.id: ist for ist in instance}
        upd_set = set()
        ret = []
        for item in validated_data:
            if 'id' in item:
                upd_set.add(item['id'])
                # 更新现有数据
                ist = ist_map.get(item['id'], None)
                if ist:
                    ret.append(self.child.update(ist, item))
            else:
                # 创建新数据
                ret.append(self.child.create(item))
        # 删除数据
        ist_del = set(ist_map.keys()) - upd_set
        for ist_id in ist_del:
            ist = ist_map[ist_id]
            ist.delete()
        return ret



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/7/30
 """
from __future__ import unicode_literals, absolute_import
import time


class Counter(object):
    """
    自增计数器
    """
    def __init__(self, start=0, size=10000000):
        self.cur_idx = start
        self.size = size

    @property
    def increase(self):
        self.cur_idx += 1
        self.cur_idx %= self.size
        return self.cur_idx

    def increase_timestamp(self, sep='_'):
        return '{0}{1}{2}'.format(
            int(time.time()),
            sep,
            self.increase
        )

simple_counter = Counter()


def regular_url(url):
    """
    规整url   url/path/
    :param url:
    :return:
    """
    if url.startswith('/'):
        url = url[1:]
    if not url.endswith('/'):
        url = '{0}/'.format(url)
    return url



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/11/2
 """
from __future__ import unicode_literals, absolute_import
from rest_framework import serializers
from djdg_core.memberbao.models import PayResult


class PayResultSerializer(serializers.ModelSerializer):
    """
    支付回调结果
    """
    class Meta:
        model = PayResult


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2017/1/17
 """
from __future__ import unicode_literals, absolute_import
import pymongo
from djdg_core.apis.mg import settings


mg = pymongo.MongoClient(
    host=settings.HOST
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/11/2
 """
from __future__ import unicode_literals, absolute_import
from djdg_core.common_api.public import common_client


def get_share_content(share_key):
    """
    获取分享模板
    :param share_key:
    :return:
    """
    res = common_client.get_resource('get', app='share/content')
    ret = res.post(data={'shareKey': share_key})
    return ret.data


def get_share_content_list(appid):
    """
    分享模板查询接口
    :param appid:
    :return:
    """
    res = common_client.get_resource('get', app='share/content')
    ret = res.post.all(data={'appid': appid})
    return ret.data


def save_share_content(share_content):
    """
    编辑分享模板
    :param share_content:
        {
            'iconUrl': '',
            'content': '',
            'linkUrl': '',
            'title': '',
            'sharekey': ''
        }
    :return:
    """
    res = common_client.get_resource('content', 'share')
    ret = res.post.save(data={'shareContent': share_content})
    return ret.data


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/11/2
 """
from __future__ import unicode_literals, absolute_import
from rest_framework import viewsets
from rest_framework import mixins
from rest_framework.response import Response
from djdg_core.memberbao import serializers
from djdg_core.rest_permissions import allow_any
from djdg_core.memberbao import signals


class PayResultViewSet(mixins.CreateModelMixin, viewsets.GenericViewSet):
    """
    支付回调结果
    """
    serializer_class = serializers.PayResultSerializer
    queryset = serializers.PayResult.objects

    @allow_any
    def create(self, request, *args, **kwargs):
        serializer = self.get_serializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        self.perform_create(serializer)
        headers = self.get_success_headers(serializer.data)
        signals.pay_result_callback_event.send(
            None,
            pay_result=serializer.data
        )
        return Response(serializer.data, headers=headers)


# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-12-02 06:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AllocateRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('request_no', models.CharField(max_length=128, verbose_name='\u8bf7\u6c42id')),
                ('notify_id', models.IntegerField(verbose_name='\u901a\u77e5id')),
                ('app_id', models.CharField(max_length=128, verbose_name='appid \u5bf9\u5e94 merchantId')),
                ('operator_id', models.PositiveSmallIntegerField(verbose_name='\u64cd\u4f5cid')),
                ('user_id', models.IntegerField(verbose_name='\u7528\u6237id\uff0c\u5bf9\u5e94userIn')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=11, verbose_name='\u91d1\u989d')),
                ('desc', models.TextField(verbose_name='\u63cf\u8ff0')),
                ('create_time', models.DateTimeField(auto_now_add=True, verbose_name='\u8bb0\u5f55\u521b\u5efa\u65f6\u95f4')),
            ],
        ),
        migrations.CreateModel(
            name='PayResult',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('notify_time', models.DateTimeField(auto_now_add=True, verbose_name='\u901a\u77e5\u65f6\u95f4')),
                ('notify_type', models.CharField(max_length=128, null=True, verbose_name='\u901a\u77e5\u7c7b\u578b')),
                ('notify_id', models.IntegerField(verbose_name='\u901a\u77e5id')),
                ('order_id', models.IntegerField(verbose_name='\u8ba2\u5355id')),
                ('total_fee', models.DecimalField(decimal_places=2, default=0, max_digits=11, verbose_name='\u8ba2\u5355\u603b\u989d')),
                ('cash_fee', models.DecimalField(decimal_places=2, default=0, max_digits=11, verbose_name='\u5b9e\u9645\u652f\u4ed8\u91d1\u989d')),
                ('coupon_fee', models.DecimalField(decimal_places=2, default=0, max_digits=11, verbose_name='\u4ee3\u91d1\u5238\u6216\u8005\u4f18\u60e0\u91d1\u989d')),
                ('coupon_count', models.IntegerField(default=0, verbose_name='\u4ee3\u91d1\u5238\u6216\u7acb\u51cf\u4f18\u60e0\u4f7f\u7528\u6570\u91cf')),
                ('pay_time', models.DateTimeField(auto_now_add=True, verbose_name='\u652f\u4ed8\u65f6\u95f4')),
                ('pay_way', models.IntegerField(null=True, verbose_name='\u652f\u4ed8\u7c7b\u578b')),
                ('app_id', models.CharField(max_length=64, null=True, verbose_name='app id')),
                ('pay_type', models.CharField(blank=True, max_length=64, null=True, verbose_name='\u652f\u4ed8\u7c7b\u578b')),
                ('create_time', models.DateTimeField(auto_now_add=True, verbose_name='\u8bb0\u5f55\u521b\u5efa\u65f6\u95f4')),
            ],
        ),
        migrations.CreateModel(
            name='RefundRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('notify_id', models.IntegerField(verbose_name='\u901a\u77e5id')),
                ('amount', models.DecimalField(decimal_places=2, max_digits=11, verbose_name='\u8ba1\u5212\u9000\u56de\u91d1\u989d')),
                ('confirm_amount', models.DecimalField(decimal_places=2, max_digits=11, verbose_name='\u5b9e\u9645\u9000\u56de\u91d1\u989d')),
                ('order_id', models.IntegerField(verbose_name='\u8ba2\u5355id')),
                ('refund_id', models.CharField(max_length=64, verbose_name='\u9000\u6b3eid\uff0c\u7531\u4f1a\u5458\u5b9d\u8fd4\u56de')),
                ('user_id', models.IntegerField(verbose_name='\u9000\u6b3e\u7528\u6237id\uff0c\u5373common api\u7684\u7528\u6237id')),
                ('create_time', models.DateTimeField(auto_now_add=True, verbose_name='\u8bb0\u5f55\u521b\u5efa\u65f6\u95f4')),
            ],
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/8/3
 """
from __future__ import unicode_literals, absolute_import
from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response
from collections import OrderedDict


class SimplePageNumberPagination(PageNumberPagination):
    """
    按页数分页
    """
    page_query_param = 'page'
    page_size_query_param = 'limit'

    def get_paginated_response(self, data):
        count = self.page.start_index() if 0 == self.page.start_index() else self.page.end_index() - self.page.start_index() + 1
        return Response(
            OrderedDict([
                ('count', count),
                ('currPage', self.page.number),
                ('totalPage', self.page.paginator.num_pages),
                ('hasNext', self.page.has_next()),
                ('total', self.page.paginator.count),
                ('data', data),
                ('next', self.get_next_link()),
                ('previous', self.get_previous_link()),
            ])
        )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/11/2
 """
from __future__ import unicode_literals, absolute_import
from django.conf.urls import url
from django.conf.urls import include
from rest_framework.routers import DefaultRouter
from djdg_core.memberbao import views


payresult_router = DefaultRouter()
payresult_router.register('payresult', views.PayResultViewSet, base_name='payresult')


urlpatterns = [
    url('^memberbao/', include(payresult_router.urls), name='payresult'),
]


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/10/17
 """
from __future__ import unicode_literals, absolute_import
from django.conf import settings
from rest_framework import viewsets
from rest_framework.parsers import MultiPartParser
from rest_framework.decorators import list_route
from rest_framework.response import Response
from djdg_core.common_api import sms
from djdg_core.common_api import authentication
from djdg_core.exceptions import ParameterNotExistOrEmptyError
from djdg_core.rest_permissions import allow_any
from djdg_core.common_api import serializers
from djdg_core.common_api import signals
from djdg_core.common_api.file import file_upload


class SmsViewSet(viewsets.ViewSet):
    """
    短信视图
    """
    @list_route(methods=['post'])
    @allow_any
    def code(self, request, *args, **kwargs):
        """
        发送短信验证码
        :param request:
            request.data = {
                'phone': '手机号码，字符串'
            }
        :param args:
        :param kwargs:
        :return:
        """
        phone = request.data.get('phone')
        if not phone:
            raise ParameterNotExistOrEmptyError('phone')
        sms.send_code(phone, content=settings.SMS_TEMPLATE)
        return Response({})

    @list_route(methods=['post'])
    @allow_any
    def voicecode(self, request, *args, **kwargs):
        """
        发送语音验证码
        :param request:
            request.data: {
                'phone': '手机号码，字符串'
            }
        :param args:
        :param kwargs:
        :return:
        """
        phone = request.data.get('phone')
        if not phone:
            raise ParameterNotExistOrEmptyError('phone')
        sms.send_voice(phone)
        return Response({})

    @list_route(methods=['post'])
    @allow_any
    def verify(self, request, *args, **kwargs):
        """
        验证验证码
        :param request:
            request.data: {
                'phone': '',
                'code': ''
            }
        :param args:
        :param kwargs:
        :return:
        """
        phone = request.data.get('phone')
        if not phone:
            raise ParameterNotExistOrEmptyError('phone')
        code = request.data.get('code')
        if not code:
            raise ParameterNotExistOrEmptyError('code')
        sms.code_verify(phone, code)
        return Response({})


class RegisterViewSet(viewsets.GenericViewSet):
    """
    注册相关接口
    """
    serializer_class = serializers.RegisterSerializer

    @list_route(methods=['post'])
    @allow_any
    def code(self, request, *args, **kwargs):
        """
        验证码方式  注册/登录接口
        :param request:
            request.data: {
                'code': '',
                'phoneNumber': '',
                'introducerCode': '',
                'unionid': ''
            }
        :param args:
        :param kwargs:
        :return:
        """
        data = request.data
        serializer = self.get_serializer(data=data)
        serializer.is_valid(raise_exception=True)
        data = serializer.data
        user = authentication.register_by_code(**data)
        signals.user_login_by_sms_code.send(
            None,
            user=user
        )
        return Response(data=user)


class FileUploadViewSet(viewsets.GenericViewSet):
    parser_classes = (MultiPartParser, )

    @list_route(methods=['post'])
    def upload(self, request, *args, **kwargs):
        """
        文件上传
        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        data = {}
        for field, file_obj in request._request.FILES.items():
            url = file_upload((file_obj.name, file_obj, file_obj.content_type))
            data[field] = url
        return Response(data=data)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/12/20
 """
from __future__ import unicode_literals, absolute_import
from django.conf.urls import include
from django.conf.urls import url
from djdg_core.core_auth import views
from rest_framework.routers import DefaultRouter
from djdg_core.core_auth.decorators import permit_all_access
from djdg_core.utils.urlpatterns import decorate_urlpatterns


user_router = DefaultRouter()
user_router.register('users', views.UserViewSet, 'user_info')


urlpatterns = [
    url('', include(user_router.urls), name='users')
]

urlpatterns = decorate_urlpatterns(urlpatterns, permit_all_access)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/10/20
 """
from __future__ import unicode_literals, absolute_import
from django import dispatch


# 支付结果回调事件
pay_result_callback_event = dispatch.Signal(providing_args=['pay_result'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2017/1/7
 """
from __future__ import unicode_literals, absolute_import
from django.conf import settings


config = settings.ES

HOSTS = config['HOSTS']
INDEX = config['INDEX']


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
author lilx
created on 2016/8/5
 """
from __future__ import unicode_literals, absolute_import
from djdg_core import settings
from django.http.response import FileResponse
from django.http.response import JsonResponse
from django.http.response import HttpResponse
from django.utils.translation import ugettext as _
from django.utils.encoding import force_unicode
from rest_framework.response import Response
from djdg_core.exceptions import StanderResponseError
from djdg_core.exceptions import rest_error
import logging
import StringIO
import re
import types
import json
from collections import OrderedDict


class LogMiddleware(object):
    """
    记录请求与响应日志，该中间件请放在第一个
    """
    default_headers = ['Content-Type', 'Content-Length']

    def __init__(self):
        self.setting = settings.MIDDLEWARE_CONFIG[self.__class__.__name__]
        self.logger = logging.getLogger(self.setting['LOGGER'])
        self.request_log_level = self.setting.get('REQUEST_LOG_LEVEL', logging.INFO)
        self.response_log_level = self.setting.get('RESPONSE_LOG_LEVEL', logging.INFO)
        self.text_max_length = self.setting.get('TEXT_MAX_LENGTH', 512)
        valuable_headers = list(self.setting.get('VALUABLE_HEADERS', []))
        self.valuable_headers = {}
        for h in (self.default_headers + valuable_headers):
            _h = h.replace('-', '_').upper()
            self.valuable_headers[_h] = h
        self.text_reg = re.compile('text|json')
        self.json_reg = re.compile('json')

    def process_request(self, request):
        """
        将请求信息写入日志
        request: from address post
        uri: http://www.thy360.com
        headers: content_type: text/plain; content_length: 22 ...
        body:
        post: k=v,
        file: file-name;file name

        :param request:
        :return:
        """
        log_cnt = StringIO.StringIO()
        log_cnt.write('request [{rid}]: from {address} {method}\n'.format(
            rid=id(request),
            address=request.META['REMOTE_ADDR'],
            method=request.method
        ))
        log_cnt.write('uri: {uri}\n'.format(uri=request.get_raw_uri()))
        headers = []
        for h, _h in self.valuable_headers.items():
            v = request.META.get(h)
            headers.append('{h}: {v}'.format(h=_h, v=v))
        log_cnt.write('headers: {headers}\n'.format(headers='; '.join(headers)))
        content_type = request.META.get('CONTENT_TYPE') or ''
        if self.text_reg.search(content_type):
            log_cnt.write('body: {body}\n'.format(body=force_unicode(request.body)))
        post_data = ['{k}={v}'.format(k=k, v=v) for k, v in request.POST.items()]
        if post_data:
            log_cnt.write('post: {post}\n'.format(post='; '.join(post_data)))
        file_data = ['{k}={v}'.format(k=k, v=v) for k, v in request.FILES.items()]
        if file_data:
            log_cnt.write('files: {files}\n'.format(files='; '.join(file_data)))
        log_cnt.seek(0)
        self.logger.log(self.request_log_level, log_cnt.read())

    def process_view(self, request, callback, callback_args, callback_kwargs):
        """
        将view信息写入日志
        :param request:
        :param callback:
        :param callback_args:
        :param callback_kwargs:
        :return:
        """
        log_cnt = StringIO.StringIO()
        log_cnt.write('request [{rid}]: from {address} {method}\n'.format(
            rid=id(request),
            address=request.META['REMOTE_ADDR'],
            method=request.method
        ))
        module_name = callback.__module__
        if type(callback) is types.FunctionType:
            class_name = None
        elif type(getattr(callback, 'im_self')) is types.TypeType:
            class_name = callback.im_self.__name__
        elif type(getattr(callback, 'im_class')) is types.TypeType:
            class_name = callback.im_class.__name__
        else:
            class_name = None
        func_name = callback.__name__
        view = '.'.join([n for n in [module_name, class_name, func_name] if n])
        log_cnt.write('view: {view}\n'.format(view=view))
        log_cnt.seek(0)
        self.logger.log(self.request_log_level, log_cnt.read())

    def process_response(self, request, response):
        """
        将响应信息写入日志
        :param request:
        :param response:
        :return:
        """
        log_cnt = StringIO.StringIO()
        log_cnt.write('response for request [{rid}]: from {address} {method}\n'.format(
            rid=id(request),
            address=request.META['REMOTE_ADDR'],
            method=request.method
        ))
        headers = {}
        if isinstance(response, FileResponse):
            content = 'is a file'
        elif isinstance(response, HttpResponse):
            for k, v in response._headers.values():
                headers[k] = v
            if hasattr(response, 'render') and callable(response.render):
                response.render()
            content = response.content
            content_type = headers.get('Content-Type', 'text/html')
            if not self.json_reg.search(content_type) and len(content) > self.text_max_length:
                content = '{0} [{1}]; content too large.'.format(content_type, len(content))
        elif isinstance(response, (dict, list, tuple)):
            content = json.dumps(response)
        else:
            content = str(response)
        if headers:
            log_cnt.write('headers: {headers}\n'.format(
                headers='; '.join([ '{0}: {1}'.format(k, v) for k, v in headers.items()])
            ))
        if content:
            content = force_unicode(content)
            log_cnt.write('content: {content}\n'.format(content=content))
        log_cnt.seek(0)
        self.logger.log(self.response_log_level, log_cnt.read())
        return response


class ResponseMiddleware(object):
    """
    处理响应的中间件
    1 将标准错误转换为json响应
    2 在标准的json响应数据中加上status与msg
    """
    def __init__(self):
        self.setting = settings.MIDDLEWARE_CONFIG.get(self.__class__.__name__, {})
        self.status_param = self.setting.get('STATUS_PARAM', 'statusCode')
        self.msg_param = self.setting.get('MSG_PARAM', 'msg')
        self.data_param = self.setting.get('DATA_PARAM', 'data')
        self.ok_status = self.setting.get('OK_STATUS', 0)
        self.ok_msg = self.setting.get('OK_MSG', 'success')
        self.default_data_value = self.setting.get('DEFAULT_DATA', {})

    def process_exception(self, request, e):
        if not isinstance(e, StanderResponseError):
            return None
        return JsonResponse(
            OrderedDict([
                (self.status_param, e.status),
                (self.msg_param, _(e.msg)),
                (self.data_param, e.data if e.data else self.default_data_value)
            ])
        )

    def process_response(self, request, response):
        """
        对rest Response，dict，tuple，list及其子类进行返回结果包装
        :param request:
        :param response:
        :return:
        """
        status_code = self.ok_status
        msg = _('success')
        if isinstance(response, Response):
            data = response.data
            if response.exception:
                status_code = rest_error.status
                msg = rest_error.msg
                if isinstance(response.data, dict):
                    msg_list = []
                    for k, v in response.data.items():
                        v = self.str(v)
                        msg_list.append('{0}: {1}'.format(k, v))
                    msg = '; '.join(msg_list) or rest_error.msg
        elif isinstance(response, (dict, tuple, list)):
            data = response
        else:
            return response
        data = OrderedDict([
            (self.status_param, status_code),
            (self.msg_param, msg),
            (self.data_param, data)
        ])
        if isinstance(response, Response):
            response.data = data
            # 重新render
            response._is_rendered = False
            response = response.render()
            return response
        return Response(data)

    def str(self, obj):
        if isinstance(obj, (list, tuple)):
            return ', '.join([self.str(o) for o in obj])
        elif isinstance(obj, dict):
            return ', '.join(['{k}={v}'.format(k=k, v=self.str(v)) for k, v in obj.items()])
        else:
            return obj
